# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from glob import glob
from sos.report.plugins import Plugin, IndependentPlugin


class Scsi(Plugin, IndependentPlugin):
    """
    Collects various information about the SCSI devices install on the host
    system.

    This plugin will capture a large amount of data from the /sys filesystem,
    as well as several different invocations of the `lsscsi` command.

    Additionally, several `sg_persist` commands will be collected for each
    SCSI device identified by sos. Note that in most cases these commands are
    provided by the `sg3_utils` package which may not be present by default.
    """

    short_desc = 'SCSI devices'

    plugin_name = 'scsi'
    profiles = ('storage', 'hardware')

    def setup(self):
        self.add_copy_spec([
            "/proc/scsi",
            "/etc/stinit.def",
            "/sys/bus/scsi",
            "/sys/class/scsi_host",
            "/sys/class/scsi_disk",
            "/sys/class/scsi_device",
            "/sys/class/scsi_generic"
        ])

        scsi_types = ["enclosu"]
        result = self.collect_cmd_output('lsscsi -g')
        if result['status'] == 0:
            for line in result['output'].splitlines():
                if line.split()[1] in scsi_types:
                    devsg = line.split()[-1]
                    self.add_cmd_output(f"sg_ses -p2 -b1 {devsg}")

        self.add_cmd_output("lsscsi -i", suggest_filename="lsscsi",
                            tags="lsscsi")

        self.add_cmd_output([
            "sg_map -x",
            "lspath",
            "lsmap -all",
            "lsnports",
            "lsscsi -H",
            "lsscsi -d",
            "lsscsi -s",
            "lsscsi -L",
            "lsscsi -iw",
            "lsscsi -t",
        ])

        scsi_hosts = glob("/sys/class/scsi_host/*")
        self.add_device_cmd("udevadm info -a %(dev)s", devices=scsi_hosts)

        self.add_device_cmd([
            "sg_persist --in -k -d %(dev)s",
            "sg_persist --in -r -d %(dev)s",
            "sg_persist --in -s -d %(dev)s",
            "sg_inq %(dev)s"
        ], devices='block', whitelist=['sd.*'])

# vim: set et ts=4 sw=4 :



# Copyright (c) 2016 Bryan Quigley <bryan.quigley@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class CanonicaLivepatch(Plugin, UbuntuPlugin):

    short_desc = 'Canonical Livepatch Service'

    plugin_name = 'canonical_livepatch'
    profiles = ('system', 'kernel')
    commands = ('canonical-livepatch',)
    services = ('snap.canonical-livepatch.canonical-livepatchd',)

    def setup(self):
        self.add_cmd_output([
            "canonical-livepatch status --verbose",
            "canonical-livepatch --version",
            "canonical-livepatch config",
        ])

    def postproc(self):
        self.do_cmd_private_sub("canonical-livepatch config")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2011, 2012 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Infiniband(Plugin, IndependentPlugin):

    short_desc = 'Infiniband information'

    plugin_name = 'infiniband'
    profiles = ('hardware',)
    packages = ('libibverbs-utils', 'opensm', 'rdma', 'infiniband-diags')

    def setup(self):
        self.add_copy_spec([
            "/etc/ofed/openib.conf",
            "/etc/ofed/opensm.conf",
            "/etc/rdma"
        ])

        self.add_copy_spec("/var/log/opensm*")

        self.add_cmd_output([
            "ibv_devices",
            "ibv_devinfo -v",
            "ibstat",
            "ibstatus",
            "ibswitches"
        ])

        # run below commands for every IB device and its active port
        ports_cmds = [
            "ibhosts",
            "iblinkinfo",
            "sminfo",
            "perfquery"
        ]
        ib_sysdir = "/sys/class/infiniband/"
        ib_devs = self.listdir(ib_sysdir) if self.path_isdir(ib_sysdir) else []
        for ibdev in ib_devs:
            # Skip OPA hardware, as infiniband-diags tools does not understand
            # OPA specific MAD sent by opa-fm. Intel provides OPA specific
            # tools for OPA fabric diagnose.
            if ibdev.startswith("hfi"):
                continue

            for port in self.listdir(ib_sysdir + ibdev + "/ports"):
                # skip IWARP and RoCE devices
                lfile = ib_sysdir + ibdev + "/ports/" + port + "/link_layer"
                try:
                    with open(lfile, 'r', encoding='UTF-8') as link_fp:
                        link_layer = link_fp.readline()
                        if link_layer != "InfiniBand\n":
                            continue
                except IOError:
                    continue

                sfile = ib_sysdir + ibdev + "/ports/" + port + "/state"
                try:
                    with open(sfile, 'r', encoding='UTF-8') as state_fp:
                        state = state_fp.readline()
                        if not state.endswith(": ACTIVE\n"):
                            continue
                except IOError:
                    continue

                opts = f"-C {ibdev} -P {port}"
                self.add_cmd_output([f"{c} {opts}" for c in ports_cmds])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc.,Poornima M. Kshirsagar <pkshiras@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import hashlib
import json
import os
from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)
from sos.policies.distros.redhat import RHELPolicy


class Python(Plugin):
    """Captures information on the installed python runtime(s), as well as
    python modules installed via pip.
    """

    short_desc = 'Python runtime'

    plugin_name = 'python'
    profiles = ('system',)

    packages = ('python',)

    python_version = "python -V"

    def setup(self):
        self.add_cmd_output(
            self.python_version, suggest_filename="python-version"
        )
        pips = self.exec_cmd("whereis pip -b")
        if pips['status'] == 0:
            # output is like:
            # pip: /usr/bin/pip2.7 /usr/bin/pip3.6
            # where we must skip the first word
            for pip in pips['output'].split()[1:]:
                self.add_cmd_output(f"{pip} -v list installed")


class UbuntuPython(Python, DebianPlugin, UbuntuPlugin):

    python_version = "python3 -V"
    packages = ('python3',)


class RedHatPython(Python, RedHatPlugin):
    """In addition to the base information, on Red Hat family distributions the
    python plugin also supports the 'hashes' option. If enabled, this plugin
    will generate a json-formatted listing of all pyfiles within the
    distribution-standard python package installation locations.
    """

    packages = ('python', 'python36', 'python2', 'python3', 'platform-python')
    option_list = [
        PluginOpt('hashes', default=False,
                  desc='collect hashes for all python files')
    ]

    def setup(self):
        self.add_cmd_output(['python2 -V', 'python3 -V'])
        if isinstance(self.policy, RHELPolicy) and \
                self.policy.dist_version() == 8:
            self.python_version = "/usr/libexec/platform-python -V"
        super().setup()

    def collect(self):
        if self.get_option('hashes'):
            with self.collection_file('digests.json') as hfile:
                hfile.write(json.dumps(self.get_hashes(), indent=4))

    def get_hashes(self):
        """ Get the hashes for Python files """
        digests = {
            'digests': []
        }
        py_paths = [
            '/usr/lib',
            '/usr/lib64',
            '/usr/local/lib',
            '/usr/local/lib64'
        ]

        for py_path in py_paths:
            for root, _, files in os.walk(self.path_join(py_path)):
                for _file in files:
                    if not _file.endswith('.py'):
                        continue
                    filepath = self.path_join(root, _file)
                    try:
                        with open(filepath, 'rb') as file:
                            digest = hashlib.sha256()
                            data = file.read(1024)
                            while data:
                                digest.update(data)
                                data = file.read(1024)

                            digest = digest.hexdigest()
                            digests['digests'].append({
                                'filepath': filepath,
                                'sha256': digest
                            })
                    except IOError:
                        self._log_error("Unable to read python file at "
                                        f"{filepath}")
        return digests

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc., Sandro Bonazzola <sbonazzo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OvirtImageIO(Plugin, RedHatPlugin):

    short_desc = 'oVirt Image I/O Daemon / Proxy'

    packages = (
        'ovirt-imageio-daemon',
        'ovirt-imageio-proxy',
    )

    plugin_name = 'ovirt_imageio'
    profiles = ('virt',)

    def setup(self):
        all_logs = self.get_option('all_logs')

        # Add configuration files
        self.add_copy_spec([
            '/etc/ovirt-imageio-daemon/logger.conf',
            '/etc/ovirt-imageio-daemon/daemon.conf',
            '/etc/ovirt-imageio-proxy/ovirt-imageio-proxy.conf',
            '/etc/ovirt-imageio/conf.d/*.conf',
        ])

        if all_logs:
            logs = ['/var/log/ovirt-imageio-proxy/image-proxy.log*',
                    '/var/log/ovirt-imageio-daemon/daemon.log*',
                    '/var/log/ovirt-imageio/daemon.log*']
        else:
            logs = ['/var/log/ovirt-imageio-proxy/image-proxy.log',
                    '/var/log/ovirt-imageio-daemon/daemon.log',
                    '/var/log/ovirt-imageio/daemon.log']

        # Add log files
        self.add_copy_spec(logs)


# vim: expandtab tabstop=4 shiftwidth=4



# Copyright (c) 2023 Canonical Ltd., Chi Wai Chan <chiwai.chan@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import json
import pathlib
import unittest
from unittest.mock import call, patch

from sos.collector.clusters.juju import _parse_option_string, juju, _get_index
from sos.options import ClusterOption


class MockOptions:

    def __init__(self):
        self.cluster_options = []


def get_juju_output(model):
    _dir = pathlib.Path(__file__).parent.resolve()
    with open(_dir / "data" / f"juju_output_{model}.json",
              encoding='utf-8') as f:
        return f.read()


def get_juju_status(cmd):
    if "-m" in cmd:
        model = cmd.split()[3]
    else:
        model = "sos"

    return {
        "status": 0,
        "output": get_juju_output(model),
    }


def get_juju_version():
    return "2.9.45"


def test_parse_option_string():
    result = _parse_option_string("    a,b,c")
    assert result == ["a", "b", "c"]

    result = _parse_option_string()
    assert result == []


class JujuTest(unittest.TestCase):
    """Test for juju cluster."""

    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_no_filter(self, mock_exec_primary_cmd):
        """No filter."""
        mock_opts = MockOptions()
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        assert not nodes

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_app_filter(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Application filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="apps",
                opt_type=str,
                value="ubuntu",
                cluster=juju.__name__,
            )
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [":0", ":2", ":3"]
        mock_exec_primary_cmd.assert_called_once_with(
            "juju status  --format json"
        )

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_app_regex_filter(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Application filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="apps",
                opt_type=str,
                value="ubuntu|nginx",
                cluster=juju.__name__,
            )
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [":0", ":2", ":3", ":4"]
        mock_exec_primary_cmd.assert_called_once_with(
            "juju status  --format json"
        )

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_model_filter_multiple_models(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Multiple model filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="models",
                opt_type=str,
                value="sos,sos2",
                cluster=juju.__name__,
            ),
        )
        mock_opts.cluster_options.append(
            ClusterOption(
                name="apps",
                opt_type=str,
                value="ubuntu",
                cluster=juju.__name__,
            ),
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [
            "sos2:0",
            "sos2:1",
            "sos:0",
            "sos:2",
            "sos:3",
        ]
        mock_exec_primary_cmd.assert_has_calls(
            [
                call("juju status -m sos --format json"),
                call("juju status -m sos2 --format json"),
            ]
        )

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_model_filter(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Model filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="models",
                opt_type=str,
                value="sos",
                cluster=juju.__name__,
            )
        )
        mock_opts.cluster_options.append(
            ClusterOption(
                name="apps",
                opt_type=str,
                value="ubuntu",
                cluster=juju.__name__,
            ),
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [
            "sos:0",
            "sos:2",
            "sos:3",
        ]
        mock_exec_primary_cmd.assert_has_calls(
            [
                call("juju status -m sos --format json"),
            ]
        )

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_unit_filter(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Node filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="units",
                opt_type=str,
                value="ubuntu/0,ubuntu/1",
                cluster=juju.__name__,
            )
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [":0", ":2"]

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_get_nodes_machine_filter(
        self, mock_exec_primary_cmd, mock_get_juju_version
    ):
        """Machine filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="machines",
                opt_type=str,
                value="0,2",
                cluster=juju.__name__,
            )
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        print(nodes)
        assert nodes == [":0", ":2"]

    @patch(
        "sos.collector.clusters.juju.juju._get_juju_version",
        side_effect=get_juju_version,
    )
    @patch(
        "sos.collector.clusters.juju.juju.exec_primary_cmd",
        side_effect=get_juju_status,
    )
    # pylint: disable=unused-argument
    def test_subordinates(self, mock_exec_primary_cmd, mock_get_juju_version):
        """Subordinate filter."""
        mock_opts = MockOptions()
        mock_opts.cluster_options.append(
            ClusterOption(
                name="apps",
                opt_type=str,
                value="nrpe",
                cluster=juju.__name__,
            )
        )
        cluster = juju(
            commons={
                "tmpdir": "/tmp",
                "cmdlineopts": mock_opts,
            }
        )
        nodes = cluster.get_nodes()
        nodes.sort()
        assert nodes == [":0", ":2", ":3"]
        mock_exec_primary_cmd.assert_called_once_with(
            "juju status  --format json"
        )


class IndexTest(unittest.TestCase):

    def test_subordinate_parent_miss_units(self):
        """Fix if subordinate's parent is missing units."""
        model = "sos"
        index = _get_index(model_name=model)

        juju_status = json.loads(get_juju_output(model=model))
        juju_status["applications"]["ubuntu"].pop("units")

        # Ensure these commands won't fall even when
        # subordinate's parent's units is missing.
        index.add_principals(juju_status)
        index.add_subordinates(juju_status)

    def test_subordinate_miss_parent(self):
        """Fix if subordinate is missing parent."""
        model = "sos"
        index = _get_index(model_name=model)

        juju_status = json.loads(get_juju_output(model=model))
        index.add_principals(juju_status)

        index.apps.pop("ubuntu")
        # Ensure command won't fall even when
        # subordinate's parent is missing
        index.add_subordinates(juju_status)


# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Sadique Puthen <sputhenp@redhat.com>
# Copyright (C) 2019 Red Hat Inc., Stepan Broz <sbroz@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, IndependentPlugin, SoSPredicate,
                                PluginOpt)


class Libreswan(Plugin, IndependentPlugin):

    short_desc = 'Libreswan IPsec'

    plugin_name = 'libreswan'
    profiles = ('network', 'security', 'openshift')
    option_list = [
        PluginOpt('ipsec-barf', default=False,
                  desc='collect ipsec barf output')
    ]

    files = ('/etc/ipsec.conf',)
    packages = ('libreswan', 'openswan')

    def setup(self):
        self.add_copy_spec([
            "/etc/ipsec.conf",
            "/etc/ipsec.d",
            "/proc/net/xfrm_stat"
        ])

        # although this is 'verification' it's normally a very quick
        # operation so is not conditional on --verify
        self.add_cmd_output([
            'ipsec verify',
            'ipsec whack --status',
            'ipsec whack --listall',
            'certutil -L -d sql:/etc/ipsec.d'
        ])

        # may load xfrm kmods
        xfrm_pred = SoSPredicate(self, kmods=['xfrm_user', 'xfrm_algo'],
                                 required={'kmods': 'all'})
        self.add_cmd_output([
            'ip xfrm policy',
            'ip xfrm state'
        ], pred=xfrm_pred)

        if self.get_option("ipsec-barf"):
            self.add_cmd_output("ipsec barf")

        self.add_forbidden_path([
            '/etc/ipsec.secrets',
            '/etc/ipsec.secrets.d',
            '/etc/ipsec.d/*.db',
            '/etc/ipsec.d/*.secrets'
        ])

    def postproc(self):
        # Remove any sensitive data.
        # "ip xfrm state" output contains encryption or authentication private
        # keys:
        xfrm_state_regexp = r'(aead|auth|auth-trunc|enc)' \
                            r'(\s.*\s)(0x[0-9a-f]+)'
        self.do_cmd_output_sub("state", xfrm_state_regexp,
                               r"\1\2********")

        if self.get_option("ipsec-barf"):
            self.do_cmd_output_sub("barf", xfrm_state_regexp,
                                   r"\1\2********")

# vim: set et ts=4 sw=4 :



# Copyright (c) 2023 Canonical Ltd., Chi Wai Chan <chiwai.chan@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import subprocess
import unittest
from unittest.mock import patch

from sos.collector.exceptions import JujuNotInstalledException
from sos.collector.transports.juju import JujuSSH


class MockCmdLineOpts:
    ssh_user = "user_abc"
    sudo_pw = "pw_abc"
    root_password = "root_pw_abc"


class JujuSSHTest(unittest.TestCase):
    def setUp(self):
        self.juju_ssh = JujuSSH(
            commons={
                "cmdlineopts": MockCmdLineOpts,
                "tmpdir": "/tmp/sos-juju/",
                "need_sudo": False,
            },
            address="model_abc:unit_abc",
        )

    @patch("sos.collector.transports.juju.subprocess.check_output")
    def test_check_juju_installed_err(self, mock_subprocess_check_output):
        """Raise error if juju is not installed."""
        mock_subprocess_check_output.side_effect = (
            subprocess.CalledProcessError(returncode="127", cmd="cmd_abc")
        )
        with self.assertRaises(JujuNotInstalledException):
            self.juju_ssh._check_juju_installed()

    @patch("sos.collector.transports.juju.subprocess.check_output")
    # pylint: disable=unused-argument
    def test_check_juju_installed_true(self, mock_subprocess_check_output):
        """Return True if juju is installed."""
        result = self.juju_ssh._check_juju_installed()
        assert result

    @patch("sos.collector.transports.juju.subprocess.check_output")
    def test_chmod(self, mock_subprocess_check_output):
        self.juju_ssh._chmod(fname="file_abc")
        mock_subprocess_check_output.assert_called_with(
            f"{self.juju_ssh.remote_exec} sudo chmod o+r file_abc",
            stderr=subprocess.STDOUT,
            shell=True,
        )

    @patch(
        "sos.collector.transports.juju.JujuSSH._check_juju_installed",
        return_value=True,
    )
    # pylint: disable=unused-argument
    def test_connect(self, mock_result):
        self.juju_ssh.connect(password=None)
        assert self.juju_ssh.connected

    def test_remote_exec(self):
        assert (
            self.juju_ssh.remote_exec == "juju ssh -m model_abc unit_abc"
        )

    @patch(
        "sos.collector.transports.juju.sos_get_command_output",
        return_value={"status": 0},
    )
    @patch("sos.collector.transports.juju.JujuSSH._chmod", return_value=True)
    # pylint: disable=unused-argument
    def test_retrieve_file(self, mock_chmod, mock_sos_get_cmd_output):
        self.juju_ssh._retrieve_file(fname="file_abc", dest="/tmp/sos-juju/")
        mock_sos_get_cmd_output.assert_called_with(
            "juju scp -m model_abc -- -r unit_abc:file_abc /tmp/sos-juju/"
        )


# vim: set et ts=4 sw=4 :



# Copyright 2023 Canonical Ltd. Arif Ali <arif.ali@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.policies.package_managers import PackageManager


class SnapPackageManager(PackageManager):
    """Subclass for snap-based distributions
    """

    query_command = "snap list"
    query_path_command = ""
    verify_command = ""
    verify_filter = ""

    def _parse_pkg_list(self, pkg_list):
        for line in pkg_list.splitlines():
            if line == "":
                continue
            pkg = line.split()
            if pkg[0] == "Name" or pkg[0] == "Connection":
                continue
            name, version = pkg[0], pkg[1]
            yield (name, version, None)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Sadique Puthen <sputhenp@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Oddjob(Plugin, RedHatPlugin):

    short_desc = 'OddJob task scheduler'

    plugin_name = 'oddjob'
    profiles = ('services', 'sysmgmt')

    files = ('/etc/oddjobd.conf',)
    packages = ('oddjob',)

    def setup(self):
        self.add_copy_spec([
            "/etc/oddjobd.conf",
            "/etc/oddjobd.conf.d",
            "/etc/dbus-1/system.d/oddjob.conf"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Mirantis, Inc., Denis Egorenko <degorenko@mirantis.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, DebianPlugin, UbuntuPlugin


class OpenStackBarbican(Plugin, DebianPlugin, UbuntuPlugin):

    short_desc = "OpenStack Barbican Secure storage service"
    plugin_name = "openstack_barbican"
    profiles = ('openstack', 'openstack_controller')

    packages = (
        'barbican-common',
        'barbican-keystone-listener',
        'barbican-worker'
    )

    requires_root = False

    def setup(self):
        self.add_copy_spec("/etc/barbican/")

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/barbican/*")
        else:
            self.add_copy_spec("/var/log/barbican/*.log")

        self.add_forbidden_path("/etc/barbican/*.pem")
        self.add_forbidden_path("/etc/barbican/alias/*")

    def postproc(self):
        protect_keys = [
            "password", "rabbit_password", "memcache_secret_key"
        ]
        self.do_file_sub(
            "/etc/barbican/barbican.conf",
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1********"
        )

        connection_keys = ["transport_url", "sql_connection"]

        join_con_keys = "|".join(connection_keys)

        self.do_path_regex_sub(
            "/etc/barbican/barbican.conf",
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6")


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, IndependentPlugin


class MvCLI(Plugin, IndependentPlugin):
    """
    The mvCLI plugin is meant for sas adapters, and collects information for
    each adapter discovered on the system.
    """

    short_desc = 'mvCLI Integrated RAID adapter information'

    plugin_name = "mvcli"
    commands = ("/opt/marvell/bin/mvcli",)

    def setup(self):

        # get list of adapters
        subcmds = [
            'info -o vd',
            'info -o pd',
            'info -o hba',
            'smart -p 0',
        ]

        self.add_cmd_output([f"/opt/marvell/bin/mvcli {s}" for s in subcmds])

# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class DellRAC(Plugin, IndependentPlugin):

    short_desc = 'Dell Remote Access Controller Administration'

    plugin_name = 'dellrac'
    profiles = ('system', 'storage', 'hardware',)
    packages = ('srvadmin-idracadm7',)

    option_list = [
        PluginOpt('debug', default=False, desc='capture support assist data')
    ]

    racadm = '/opt/dell/srvadmin/bin/idracadm7'
    prefix = 'idracadm7'

    def setup(self):
        for subcmd in ['getniccfg', 'getsysinfo']:
            self.add_cmd_output(
                f'{self.racadm} {subcmd}',
                suggest_filename=f'{self.prefix}_{subcmd}')

        if self.get_option("debug"):
            self.do_debug()

    def do_debug(self):
        """
        Ensure the sos_commands/dellrac directory does exist in either case
        as we will need to run the command at that dir, and also ensure
        logpath is properly populated in either case as well.
        """
        try:
            logpath = self.get_cmd_output_path()
        except FileExistsError:
            logpath = self.get_cmd_output_path(make=False)
        subcmd = 'supportassist collect -f'
        self.add_cmd_output(
            f'{self.racadm} {subcmd} support.zip',
            runat=logpath, suggest_filename=f'{self.prefix}_{subcmd}')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Memory(Plugin, IndependentPlugin):

    short_desc = 'Memory configuration and use'

    plugin_name = 'memory'
    profiles = ('system', 'hardware', 'memory')

    def setup(self):
        self.add_copy_spec([
            "/proc/pci",
            "/proc/meminfo",
            "/proc/vmstat",
            "/proc/swaps",
            "/proc/slabinfo",
            "/proc/pagetypeinfo",
            "/proc/vmallocinfo",
            "/sys/kernel/mm/ksm",
            "/sys/kernel/mm/transparent_hugepage",
            "/sys/kernel/mm/hugepages",
            "/sys/kernel/mm/lru_gen/enabled",
            "/sys/kernel/mm/lru_gen/min_ttl_ms",
        ])
        self.add_cmd_output("free", root_symlink="free")
        self.add_cmd_output([
            "free -m",
            "swapon --bytes --show",
            "swapon --summary --verbose",
            "lsmem -a -o RANGE,SIZE,STATE,REMOVABLE,ZONES,NODE,BLOCK"
        ])

        # slabtop -o will hang if not handed a tty via stdin
        self.add_cmd_output("slabtop -o", foreground=True)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class PercCLI2(Plugin, IndependentPlugin):

    short_desc = 'PowerEdge RAID Controller 2 management'

    plugin_name = 'perccli2'
    profiles = ('system', 'storage', 'hardware',)
    packages = ('perccli2',)

    option_list = [
        PluginOpt('json', default=False, desc='collect data in JSON format')
    ]

    def setup(self):
        cmd = '/opt/MegaRAID/perccli2/perccli2'
        subcmds = [
            'show ctrlcount',
            '/call show AliLog',
            '/call show all',
            '/call show termlog',
            '/call/bbu show all',
            '/call/cv show all',
            '/call/dall show',
            '/call/eall show all',
            '/call/eall/sall show all',
            '/call/sall show all',
            '/call/vall show all',
        ]

        json = ' J' if self.get_option('json') else ''

        logpath = self.get_cmd_output_path()

        for subcmd in subcmds:
            self.add_cmd_output(
                f"{cmd} {subcmd}{json}",
                suggest_filename=f"perccli2_{subcmd}{json}",
                runat=logpath)

        # /call show events need 'file=' option to get adapter info like below
        # "Adapter: # - Number of Events: xxx".
        subcmd = '/call show events'
        self.add_cmd_output(
            f"{cmd} {subcmd} file=/dev/stdout{json}",
            suggest_filename=f"perccli2_{subcmd}{json}",
            runat=logpath)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class OMSA(Plugin, IndependentPlugin):

    short_desc = 'Dell OpenManage Server Administrator (OMSA)'

    plugin_name = 'omsa'
    profiles = ('hardware', 'debug')

    files = ('/opt/dell/srvadmin/bin/omreport',)
    packages = ('srvadmin-omacore',)

    omreport = '/opt/dell/srvadmin/bin/omreport'

    def setup(self):
        self.add_copy_spec([
            "/var/log/dell/updatepackage/log/support",
            "/opt/dell/srvadmin/var/log/openmanage/Inventory.xml*",
            "/opt/dell/srvadmin/etc/omreg.cfg",
            "/opt/dell/srvadmin/etc/openmanage/oma/ini",
            "/opt/dell/srvadmin/etc/srvadmin-deng/ini",
            "/opt/dell/srvadmin/etc/srvadmin-isvc/ini/d*ini",
        ])

        self.add_cmd_output([
            f"{self.omreport} system alertaction",
            f"{self.omreport} system alertlog",
            f"{self.omreport} system cmdlog",
            f"{self.omreport} system pedestinations",
            f"{self.omreport} system platformevents",
            f"{self.omreport} system summary",
            f"{self.omreport} system events",
            f"{self.omreport} chassis info",
            f"{self.omreport} chassis biossetup",
            f"{self.omreport} storage controller",
        ], timeout=30)

# vim: et ts=4 sw=4



# Copyright Red Hat 2020, Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from shlex import quote
from sos.collector.clusters import Cluster


class satellite(Cluster):
    """
    This profile is specifically for Red Hat Satellite 6, and not earlier
    releases of Satellite.

    While note technically a 'cluster' in the traditional sense, Satellite
    does provide for 'capsule' nodes which is what this profile aims to
    enumerate beyond the 'primary' Satellite system.
    """

    cluster_name = 'Red Hat Satellite 6'
    packages = ('satellite', 'satellite-installer')

    def _psql_cmd(self, query):
        _cmd = "su postgres -c %s"
        _dbcmd = "psql foreman -c %s"
        return _cmd % quote(_dbcmd % quote(query))

    def get_nodes(self):
        cmd = self._psql_cmd('copy (select name from smart_proxies) to stdout')
        res = self.exec_primary_cmd(cmd, need_root=True)
        if res['status'] == 0:
            nodes = [
                n.strip() for n in res['output'].splitlines()
                if 'could not change directory' not in n
            ]
            return nodes
        return []

    def set_node_label(self, node):
        if node.address == self.primary.address:
            return 'satellite'
        return 'capsule'

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc.,
# Pablo Iranzo Gomez <Pablo.Iranzo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from glob import glob
from sos.report.plugins import Plugin, RedHatPlugin


class Monit(Plugin, RedHatPlugin):

    short_desc = 'Monit monitoring daemon'
    packages = ('monit',)
    profiles = ('system',)
    plugin_name = 'monit'

    # Define configuration files
    monit_conf = glob("/etc/monit.d/*")
    monit_conf.append("/etc/monit.conf")
    monit_conf.append("/etc/monitrc")

    # Define log files
    monit_log = ["/var/log/monit.log"]

    def setup(self):
        self.add_cmd_output("monit status")
        self.add_copy_spec(self.monit_log + self.monit_conf)

    def postproc(self):
        # Post process the files included to scrub any
        # password or other sensitive data

        # usernames and emails are cleaned to not disclose any
        # confidential data

        for file in self.monit_conf:
            # Remove username:password from files
            self.do_file_sub(file,
                             r"(allow) (.*):(.*)",
                             r"\1 ********:********"
                             )
            # Remove MAILSERVER username/password
            self.do_file_sub(file,
                             r"(username|password) (\w)+",
                             r"\1 ********"
                             )

# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin


class CephAnsible(Plugin, RedHatPlugin, DebianPlugin):

    short_desc = 'CEPH distributed storage - Ansible installer'

    plugin_name = 'ceph_ansible'
    profiles = ('storage', 'ceph')

    packages = ('ceph-ansible',)

    def setup(self):
        self.add_copy_spec([
            "/usr/share/ceph-ansible/group_vars/",
            "/usr/share/ceph-ansible/site*.yml",
            "/usr/share/ceph-ansible/ansible.cfg"
        ])

        self.add_forbidden_path("/usr/share/ceph-ansible/group_vars/*.sample")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import re
import json
from sos_tests import StageTwoReportTest

ARCHIVE = 'sosreport-cleanertest-2021-08-03-qpkxdid'


class ExistingArchiveCleanTest(StageTwoReportTest):
    """Ensure that we can extract an already created archive and clean it the
    same as we would an in-line run of `report --clean`.

    Note that this copies heavily from the full_report_run test.

    :avocado: tags=stagetwo
    """

    sos_cmd = f'tests/test_data/{ARCHIVE}.tar.xz'
    sos_component = 'clean'

    def test_obfuscation_log_created(self):
        self.assertFileExists(
            os.path.join(self.tmpdir, f'{ARCHIVE}-obfuscation.log')
        )

    def test_archive_type_correct(self):
        with open(os.path.join(
                self.tmpdir,
                f'{ARCHIVE}-obfuscation.log'), 'r', encoding='utf-8') as log:
            for line in log:
                if f"Loaded {ARCHIVE}" in line:
                    assert \
                        'as type sos report archive' in line, \
                        f"Incorrect archive type detected: {line}"
                    break

    def test_from_cmdline_logged(self):
        with open(os.path.join(
                self.tmpdir,
                f'{ARCHIVE}-obfuscation.log'), 'r', encoding='utf-8') as log:
            for line in log:
                if 'From cmdline' in line:
                    assert \
                        'From cmdline: True' in line, \
                        "Did not properly log cmdline run"
                    break

    def test_extraction_completed_successfully(self):
        with open(os.path.join(
                self.tmpdir,
                f'{ARCHIVE}-obfuscation.log'), 'r', encoding='utf-8') as log:
            for line in log:
                if 'Extracted path is' in line:
                    path = line.split('Extracted path is')[-1].strip()
                    assert \
                        path.startswith(self.tmpdir), \
                        (f"Extracted path appears wrong: {path} "
                         f"(tmpdir: {self.tmpdir})")
                    return
            self.fail("Extracted path not logged")

    def test_private_map_was_generated(self):
        self.assertOutputContains(
            'A mapping of obfuscated elements is available at'
        )
        map_file = re.findall(
            '/.*sosreport-.*-private_map', self.cmd_output.stdout)[-1]
        self.assertFileExists(map_file)

    def test_tarball_named_obfuscated(self):
        self.assertTrue('obfuscated' in self.archive)

    def test_hostname_not_in_any_file(self):
        # much faster to just use grep here
        content = self.grep_for_content('cleanertest')
        if not content:
            assert True
        else:
            self.fail("Hostname appears in files: %s"
                      % "\n".join(f for f in content))

    def test_no_empty_obfuscations(self):
        # get the private map file name
        map_file = re.findall(
            '/.*sosreport-.*-private_map', self.cmd_output.stdout
        )[-1]
        with open(map_file, 'r', encoding='utf-8') as mf:
            map_json = json.load(mf)
        for mapping in map_json:
            for key, val in map_json[mapping].items():
                assert key, f"Empty key found in {mapping}"
                assert val, f"{mapping} mapping for '{key}' empty"

    def test_ip_not_in_any_file(self):
        content = self.grep_for_content('10.0.0.15')
        if not content:
            assert True
        else:
            new_content = "\n".join(f for f in content)
            self.fail(f'IP appears in files: {new_content}')

    def test_user_is_obfuscated(self):
        """Ensure that the 'testuser1' user created at install is obfuscated
        """
        self.assertFileNotHasContent(
            'var/log/anaconda/journal.log',
            'testuser1'
        )


class ExistingArchiveCleanTmpTest(StageTwoReportTest):
    """Continuation of above tests which requires cleaning var / tmp keywords

    Note that this copies heavily from the full_report_run test.

    :avocado: tags=stagetwo
    """

    sos_cmd = f'--keywords avocado,ExistingArchiveCleanTmpTest --no-update \
                --disable-parsers ip,ipv6,mac,username \
                tests/test_data/{ARCHIVE}.tar.xz'
    sos_component = 'clean'

    def test_sys_tmp_not_obfuscated(self):
        """ Ensure that keywords avocado and ExistingArchiveCleanTmpTest
        remains in the final archive path despite they are parts of the
        --tmp-dir (set like
        /var/tmp/avocado_1m9g7qt1sos_tests.py.ExistingArchiveCleanTmpTest )
        """
        self.assertTrue(
            self.archive.startswith(os.getenv('AVOCADO_TESTS_COMMON_TMPDIR'))
        )



# Copyright (C) 2016 Red Hat, Inc., Sachin Patil <sacpatil@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, PluginOpt


class DNFPlugin(Plugin, RedHatPlugin):
    """
    The DNF plugin collects information for the dnf package manager and how it
    is configured for local system.

    By default, this plugin will collect configuration files from /etc/dnf,
    repo files defined in /etc/yum.repos.d/, module information, and various
    'dnf list' commands.

    When using the 'history-info' option, detailed transaction information will
    be collected for the most recent 50 dnf transactions, and will be saved to
    the sos_commands/dnf/history-info directory.
    """

    short_desc = 'dnf package manager'
    plugin_name = "dnf"
    profiles = ('system', 'packagemanager', 'sysmgmt')

    files = ('/etc/dnf/dnf.conf',)
    packages = ('dnf',)

    option_list = [
        PluginOpt('history-info', default=False,
                  desc='collect detailed transaction history')
    ]

    def get_modules_info(self, modules):
        """ Get DN module information """
        if not modules:
            return
        # take just lines with the module names, i.e. containing "[i]" and
        # not the "Hint: [d]efault, [e]nabled, [i]nstalled,.."
        for line in modules.splitlines():
            if "[i]" in line:
                module = line.split()[0]
                if module != "Hint:":
                    self.add_cmd_output("dnf module info " + module,
                                        tags='dnf_module_info')

    def setup(self):

        self.add_file_tags({
            '/etc/dnf/modules.d/.*.module': 'dnf_modules'
        })

        self.add_copy_spec([
            "/etc/dnf/",
            "/etc/yum.conf",
            "/etc/yum/pluginconf.d/",
            "/etc/yum/vars/",
        ])
        self.add_copy_spec("/etc/yum.repos.d/",
                           tags=['yum_repos_d', 'dnf_repos_d', 'dnf_repo'])

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/dnf.*")
        else:
            self.add_copy_spec("/var/log/dnf.log*")
            self.add_copy_spec("/var/log/dnf.librepo.log*")
            self.add_copy_spec("/var/log/dnf.rpm.log*")

        self.add_cmd_output("dnf module list",
                            tags='dnf_module_list')

        self.add_cmd_output([
            "dnf --version",
            "dnf list extras",
            "dnf updateinfo info security",
            "dnf updateinfo list --available",
            "package-cleanup --dupes",
            "package-cleanup --problems",
        ])

        self.add_cmd_output("dnf list installed",
                            tags=["yum_list_installed", "dnf_list_installed"])

        self.add_cmd_output('dnf -C repolist',
                            tags=['yum_repolist', 'dnf_repolist'])

        self.add_cmd_output('dnf -C repolist --verbose')

        self.add_forbidden_path([
            "/etc/pki/entitlement/key.pem",
            "/etc/pki/entitlement/*-key.pem"
        ])

        self.add_copy_spec([
            "/etc/pki/product/*.pem",
            "/etc/pki/consumer/cert.pem",
            "/etc/pki/entitlement/*.pem"
        ])

        if not self.get_option("history-info"):
            self.add_cmd_output("dnf history", tags='dnf_history')
        else:
            history = self.collect_cmd_output("dnf history",
                                              tags='dnf_history')
            transactions = -1
            if history['output']:
                for line in history['output'].splitlines():
                    try:
                        transactions = int(line.split('|')[0].strip())
                        break
                    except ValueError:
                        # not a valid line to extract transactions from, ignore
                        pass
            for tr_id in range(1, min(transactions+1, 50)):
                self.add_cmd_output(f"dnf history info {tr_id}",
                                    subdir="history-info",
                                    tags='dnf_history_info')

        # Get list of dnf installed modules and their details.
        module_cmd = "dnf module list --installed"
        modules = self.collect_cmd_output(module_cmd)
        self.get_modules_info(modules['output'])

    def postproc(self):
        # Scrub passwords in repositories and yum/dnf variables
        # Example of scrubbing:
        #
        #   password=hackme
        # To:
        #   password=********
        #
        # Whitespace around '=' is allowed.
        regexp = r"(password(\s)*=(\s)*)(\S+)\n"
        repl = r"\1********\n"
        for file in ["/etc/yum.repos.d/*", "/etc/dnf/vars/*"]:
            self.do_path_regex_sub(file, regexp, repl)

        # Scrub password and proxy_password from /etc/dnf/dnf.conf.
        # This uses the same regex patterns as above.
        #
        # Example of scrubbing:
        #
        #   proxy_password = hackme
        # To:
        #   proxy_password = ********
        #
        self.do_file_sub("/etc/dnf/dnf.conf", regexp, repl)

        # Scrub credentials in http URIs
        self.do_paths_http_sub([
            '/etc/yum.repos.d/*',
            '/var/log/dnf.*',
        ])

# vim: set et ts=4 sw=4 :



# Copyright 2023 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.cleaner.preppers import SoSPrepper


class MacPrepper(SoSPrepper):
    """
    Prepper for sourcing the host's MAC address in order to prep the mapping.
    """

    name = 'mac'

    def _get_mac_file_list(self, archive):
        if archive.is_sos:
            return ['sos_commands/networking/ip_-d_address']
        if archive.is_insights:
            return ['data/insights_commands/ip_addr']
        return []

# vim: set et ts=4 sw=4 :



# Copyright (C) 2023 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Rhc(Plugin, RedHatPlugin):

    """
    RHC is a client tool and daemon that connects the system
    to Red Hat hosted services enabling system and
    subscription management. This plugin captures
    configuration files and the output of 'rhc status'.
    """
    short_desc = 'Red Hat client for remote host configured services'

    plugin_name = "rhc"
    packages = ("rhc", )

    def setup(self):
        self.add_copy_spec([
            "/etc/rhc/*",
            "/var/log/rhc-worker-playbook",
        ])

        self.add_cmd_output([
            "rhc status",
        ])

    def postproc(self):
        # hide workers/foreman_rh_cloud.toml FORWARDER_PASSWORD
        # Example for scrubbing FORWARDER_PASSWORD
        #
        # "FORWARDER_PASSWORD=F0rW4rd3RPassW0rD"
        #
        # to
        #
        # "FORWARDER_PASSWORD= ********

        self.do_path_regex_sub("/etc/rhc/workers/foreman_rh_cloud.toml",
                               r"(FORWARDER_PASSWORD\s*=\s*)(.+)(\"\,)",
                               r"\1********\3")

        # hide ssh host keys from rhc-worker ansible playbooks
        # Example for scrubbing one of the ssh keys
        #
        # "ansible_ssh_host_key_ecdsa_public": "ABCDEFGHIJ",
        #
        # to
        #
        # "ansible_ssh_host_key_ecdsa_public": ********,

        path = "/var/log/rhc-worker-playbook/ansible/*"
        regexp = r"(\s*\"ansible_ssh_host_key_)(.+)(_public\":\s*)(.+)(\,|$)"
        self.do_path_regex_sub(path, regexp,
                               r"\1\2\3********\5")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Libraries(Plugin, IndependentPlugin):

    short_desc = 'Dynamic shared libraries'

    plugin_name = 'libraries'
    profiles = ('system',)

    option_list = [
        PluginOpt('ldconfigv', default=False,
                  desc='collect verbose ldconfig output')
    ]

    def setup(self):
        self.add_copy_spec(["/etc/ld.so.conf", "/etc/ld.so.conf.d"])
        if self.get_option("ldconfigv"):
            self.add_cmd_output("ldconfig -v -N -X")

        self.add_env_var([
            'PATH',
            'LD_LIBRARY_PATH',
            'LD_PRELOAD'
        ])

        ldconfig = self.collect_cmd_output("ldconfig -p -N -X")

        if ldconfig['status'] == 0:
            # Collect library directories from ldconfig's cache
            dirs = set()
            for lib in ldconfig['output'].splitlines():
                fqlib = lib.split(" => ", 2)
                if len(fqlib) != 2:
                    continue
                dirs.add(fqlib[1].rsplit('/', 1)[0])

            if dirs:
                self.add_dir_listing(
                    f"{' '.join(dirs)}",
                    suggest_filename='ld_so_cache'
                )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Saphana(Plugin, RedHatPlugin):

    short_desc = 'SAP HANA'
    plugin_name = 'saphana'
    profiles = ('sap',)
    files = ('/hana',)

    def setup(self):

        sids = []

        if self.path_isdir("/hana/shared"):
            shared = self.listdir("/hana/shared")
            for sid in shared:
                if len(sid) == 3:
                    sid = sid.strip()
                    sids.append(sid)

            for sid in sids:
                sidadm = f'{sid.lower()}adm'
                prefix = f'su - {sidadm} -c'

                self.add_cmd_output(f'{prefix} "HDB info"',
                                    suggest_filename=f"{sid}_HDB_info")

                self.add_cmd_output(f'{prefix} "hdbsrvutil -v"',
                                    suggest_filename=f"{sid}_version")

                self.add_cmd_output(f'{prefix} \'hdbcons "mm l -s -S -p"\'',
                                    suggest_filename=f"{sid}_memusage")

                self.add_cmd_output(f'{prefix} \'hdbcons -e hdbindexserver \
                                    "replication info"\'',
                                    suggest_filename=f"{sid}_replicainfo")

                if self.path_isdir(f"/hana/shared/{sid}/"):
                    for inst in self.listdir(f"/hana/shared/{sid}/"):
                        if "HDB" in inst:
                            inst = inst.strip()[-2:]
                            self.get_inst_info(sid, sidadm, inst)

    def get_inst_info(self, sid, sidadm, inst):
        """ Collect the given instance info """
        proc_cmd = 'su - %s -c "sapcontrol -nr %s -function GetProcessList"'
        status_fname = f"{sid}_{inst}_status"
        self.add_cmd_output(
            proc_cmd % (sidadm, inst),
            suggest_filename=status_fname
        )

        path = f"/usr/sap/{sid}/HDB{inst}/exe/python_support"
        if self.path_isdir(path):
            py_cmd = 'su - %s -c "python %s/landscapeHostConfiguration.py"'
            py_fname = f"{sid}_{inst}_landscapeConfig"
            self.add_cmd_output(
                py_cmd % (sidadm, path),
                suggest_filename=py_fname
            )

# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import UbuntuPlugin
from sos.policies.distros.debian import DebianPolicy

from sos.policies.package_managers.snap import SnapPackageManager
from sos.policies.package_managers.dpkg import DpkgPackageManager
from sos.policies.package_managers import MultiPackageManager


class UbuntuPolicy(DebianPolicy):
    vendor = "Canonical"
    vendor_urls = [
        ('Community Website', 'https://www.ubuntu.com/'),
        ('Commercial Support', 'https://www.canonical.com')
    ]
    os_release_name = 'Ubuntu'
    os_release_file = ''
    PATH = "/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games" \
           + ":/usr/local/sbin:/usr/local/bin:/snap/bin"
    _upload_url = "https://files.support.canonical.com/uploads/"
    _upload_user = "ubuntu"
    _upload_password = "ubuntu"
    _upload_method = 'put'

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

        self.package_manager = MultiPackageManager(
            primary=DpkgPackageManager,
            fallbacks=[SnapPackageManager],
            chroot=self.sysroot,
            remote_exec=remote_exec)

        try:
            if self.package_manager.pkg_by_name(
                    'sosreport')['pkg_manager'] == 'snap':
                self.sos_bin_path = '/snap/bin'
        except TypeError:
            # Use the default sos_bin_path
            pass

        self.valid_subclasses += [UbuntuPlugin]

    def dist_version(self):
        """ Returns the version stated in DISTRIB_RELEASE
        """
        try:
            with open('/etc/lsb-release', 'r', encoding='utf-8') as fp:
                lines = fp.readlines()
                for line in lines:
                    if "DISTRIB_RELEASE" in line:
                        return float(line.split("=")[1].strip())
            return False
        except (IOError, ValueError):
            return False


# vim: set et ts=4 sw=4 :



# Copyright (C) 2022 Red Hat, Inc.

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


MISTRAL_DIRECTORIES = [
        '/var/log/mistral/',
        '/var/lib/mistral/',
        ]
MISTRAL_LOGS = [
        '/var/log/mistral/*.log',
        '/var/lib/mistral/*/*.log',
        ]


class OpenStackMistral(Plugin, RedHatPlugin):
    '''Gather Mistral directories content, both data from /var/lib/mistral
    and its log from /var/log/mistral if it exists (older OSP).
    The data also embed logs for the ansible runs launched via the service,
    meaning we'll be able to properly debug failures therein. The rest of the
    data are the generated environment files, also really useful in order
    to debug an issue at deploy or day-2 operations.
    We filter out on the presence of any "mistral" related container on the
    host - usually the Undercloud presents mistral_engine, mistral_executor
    and mistral_api.
    '''

    short_desc = 'OpenStack Mistral'

    plugin_name = "openstack_mistral"
    profiles = ('openstack', 'openstack_undercloud')
    containers = ('.*mistral_engine',)

    def setup(self):
        if self.get_option('all_log'):
            self.add_copy_spec(MISTRAL_DIRECTORIES)
        else:
            self.add_copy_spec(MISTRAL_LOGS)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import pwd
import re
import inspect
from subprocess import Popen, PIPE, STDOUT
import logging
import fnmatch
import errno
import shlex
import glob
import tempfile
import threading
import time
import io
import mmap
from contextlib import closing
from collections import deque

try:
    from packaging.version import parse as parse_version
except ImportError:
    from pkg_resources import parse_version

log = logging.getLogger('sos')

# try loading magic>=0.4.20 which implements detect_from_filename method
magic_mod = False
try:
    import magic
    magic.detect_from_filename(__file__)
    magic_mod = True
except (ImportError, AttributeError):
    from textwrap import fill
    msg = """\
WARNING: Failed to load 'magic' module version >= 0.4.20 which sos aims to \
use for detecting binary files. A less effective method will be used. It is \
recommended to install proper python3-magic package with the module.
"""
    log.warning(f'\n{fill(msg, 72, replace_whitespace=False)}\n')


TIMEOUT_DEFAULT = 300

__all__ = [
    'TIMEOUT_DEFAULT',
    'ImporterHelper',
    'SoSTimeoutError',
    'TempFileUtil',
    'bold',
    'file_is_binary',
    'fileobj',
    'find',
    'get_human_readable',
    'grep',
    'import_module',
    'is_executable',
    'listdir',
    'parse_version',
    'path_exists',
    'path_isdir',
    'path_isfile',
    'path_islink',
    'path_join',
    'recursive_dict_values_by_key',
    'shell_out',
    'sos_get_command_output',
    'tac_logs',
    'tail',
]


def format_version_to_pep440(ver):
    """ Convert the version into a PEP440 compliant version scheme."""
    public_version_re = re.compile(
            r"^([0-9][0-9.]*(?:(?:a|b|rc|.post|.dev)[0-9]+)*)\+?"
            )
    try:
        _, public, local = public_version_re.split(ver, maxsplit=1)
        if not local:
            return ver
        sanitized_local = re.sub("[+~]+", ".", local).strip("-")
        pep440_version = f"{public}+{sanitized_local}"
        return pep440_version
    except Exception as err:
        log.debug(f"Unable to format {ver} to pep440 format: {err}")
        return ver


def sos_parse_version(ver, pep440=True):
    """ Converts the version to PEP440 format before parsing """
    if pep440:
        ver_pep440 = format_version_to_pep440(ver)
        return parse_version(ver_pep440)

    return parse_version(ver)


def tail(filename, number_of_bytes):
    """Returns the last number_of_bytes of filename"""
    with open(filename, "rb") as f:
        if os.stat(filename).st_size > number_of_bytes:
            f.seek(-number_of_bytes, 2)
        return f.read()


def fileobj(path_or_file, mode='r'):
    """Returns a file-like object that can be used as a context manager"""
    if isinstance(path_or_file, str):
        try:
            return open(path_or_file, mode, encoding='utf-8')
        except IOError:
            log.debug(f"fileobj: {path_or_file} could not be opened")
            return closing(io.StringIO())
    else:
        return closing(path_or_file)


def convert_bytes(num_bytes):
    """Converts a number of bytes to a shorter, more human friendly format"""
    sizes = {'T': 1 << 40, 'G': 1 << 30, 'M': 1 << 20, 'K': 1 << 10}
    for symbol, size in sizes.items():
        if num_bytes >= size:
            return f"{float(num_bytes) / size:.1f}{symbol}"
    return f"{num_bytes}"


def file_is_binary(fname):
    """Helper to determine if a given file contains binary content or not.

    This is especially helpful for `sos clean`, which cannot obfuscate binary
    data and instead, by default, will remove binary files.

    :param fname:   The full path of the file to check binaryness of
    :type fname:    ``str``

    :returns:   True if binary, else False
    :rtype:     ``bool``
    """
    if magic_mod:
        try:
            _ftup = magic.detect_from_filename(fname)
            _mimes = ['text/', 'inode/']
            return (
                _ftup.encoding == 'binary' and not
                any(_ftup.mime_type.startswith(_mt) for _mt in _mimes)
            )
        except Exception:
            pass
    # if for some reason the above check fails or magic>=0.4.20 is not present,
    # fail over to checking the very first byte of the file content
    with open(fname, 'tr', encoding='utf-8') as tfile:
        try:
            # when opened as above (tr), reading binary content will raise
            # an exception
            tfile.read(1)
            return False
        except UnicodeDecodeError:
            return True


def find(file_pattern, top_dir, max_depth=None, path_pattern=None):
    """Generator function to find files recursively.
    Usage::

        for filename in find("*.properties", "/var/log/foobar"):
            print filename
    """
    if max_depth:
        base_depth = os.path.dirname(top_dir).count(os.path.sep)
        max_depth += base_depth

    for path, dirlist, filelist in os.walk(top_dir):
        if max_depth and path.count(os.path.sep) >= max_depth:
            del dirlist[:]

        if path_pattern and not fnmatch.fnmatch(path, path_pattern):
            continue

        for name in fnmatch.filter(filelist, file_pattern):
            yield os.path.join(path, name)


def grep(pattern, *files_or_paths):
    """Returns lines matched in fnames, where fnames can either be pathnames to
    files to grep through or open file objects to grep through line by line"""
    matches = []

    for fop in files_or_paths:
        with fileobj(fop) as fo:
            matches.extend((line for line in fo if re.match(pattern, line)))

    return matches


def is_executable(command, sysroot=None):
    """Returns if a command matches an executable on the PATH"""

    paths = os.environ.get("PATH", "").split(os.path.pathsep)
    candidates = [command] + [os.path.join(p, command) for p in paths]
    if sysroot:
        candidates += [
            os.path.join(sysroot, c.lstrip('/')) for c in candidates
        ]
    return any(os.access(path, os.X_OK) for path in candidates)


def scrub_url_credential(url: str):
    """
    Replace username:password@ with ********@ in proxy URL if present
    """
    from urllib.parse import urlparse, urlunparse
    try:
        parsed_url = urlparse(url)
        if parsed_url.username or parsed_url.password:
            netloc = "********@"
            if parsed_url.hostname:
                netloc += parsed_url.hostname
            if parsed_url.port:
                netloc += f":{parsed_url.port}"
            return urlunparse((
                parsed_url.scheme, netloc, parsed_url.path,
                parsed_url.params, parsed_url.query, parsed_url.fragment
            ))
        return url
    except Exception:  # pylint: disable=broad-except
        return url


def sos_get_command_output(command, timeout=TIMEOUT_DEFAULT, stderr=False,
                           chroot=None, chdir=None, env=None, foreground=False,
                           binary=False, sizelimit=None, poller=None,
                           to_file=False, tac=False, runas=None):
    # pylint: disable=too-many-locals,too-many-branches
    """Execute a command and return a dictionary of status and output,
    optionally changing root or current working directory before
    executing command.
    """
    # Change root or cwd for child only. Exceptions in the prexec_fn
    # closure are caught in the parent (chroot and chdir are bound from
    # the enclosing scope).
    def _child_prep_fn():
        if chroot and chroot != '/':
            os.chroot(chroot)
        if runas:
            os.setgid(pwd.getpwnam(runas).pw_gid)
            os.setuid(pwd.getpwnam(runas).pw_uid)
            os.chdir(pwd.getpwnam(runas).pw_dir)
        if chdir:
            os.chdir(chdir)

    def _check_poller(proc):
        if poller() or proc.poll() == 124:
            proc.terminate()
            raise SoSTimeoutError
        time.sleep(0.01)

    if runas:
        try:
            pwd_user = pwd.getpwnam(runas)
        except KeyError:  # no such user
            return {'status': 127, 'output': "", 'truncated': ''}
        env.update({
            'HOME': pwd_user.pw_dir,
            'LOGNAME': runas,
            'PWD': pwd_user.pw_dir,
            'USER': runas
        })

    cmd_env = os.environ.copy()
    # ensure consistent locale for collected command output
    cmd_env['LC_ALL'] = 'C.UTF-8'
    # optionally add an environment change for the command
    if env:
        for key, value in env.items():
            if value:
                cmd_env[key] = value
            else:
                cmd_env.pop(key, None)
    # use /usr/bin/timeout to implement a timeout
    if timeout and is_executable("timeout"):
        command = (f"timeout {'--foreground' if foreground else ''} {timeout}s"
                   f" {command}")

    args = shlex.split(command)
    # Expand arguments that are wildcard root paths.
    expanded_args = []
    for arg in args:
        if arg.startswith("/") and "*" in arg:
            expanded_arg = glob.glob(arg)
            if expanded_arg:
                expanded_args.extend(expanded_arg)
            else:
                expanded_args.append(arg)
        else:
            expanded_args.append(arg)
    if to_file:
        if sizelimit:
            # going to use HeadReader
            _output = PIPE
        elif tac:
            # no limit but we need an intermediate file
            _output = tempfile.TemporaryFile(dir=os.path.dirname(to_file))
        else:
            # pylint: disable=consider-using-with
            _output = open(to_file, 'wb')
    else:
        _output = PIPE
    try:
        with Popen(expanded_args, shell=False, stdout=_output,
                   stderr=STDOUT if stderr else PIPE,
                   bufsize=-1, env=cmd_env, close_fds=True,
                   preexec_fn=_child_prep_fn) as p:

            if to_file:
                if sizelimit:
                    if tac:
                        _output = tempfile.TemporaryFile(
                            dir=os.path.dirname(to_file)
                        )
                    else:
                        # pylint: disable=consider-using-with
                        _output = open(to_file, 'wb')
                    reader = HeadReader(p.stdout, _output, sizelimit, binary)
                else:
                    reader = FakeReader(p, binary)
            else:
                reader = TailReader(p.stdout, sizelimit, binary)

            if poller:
                while reader.running:
                    _check_poller(p)
            else:
                try:
                    # override timeout=0 to timeout=None, as Popen will treat
                    # the former as a literal 0-second timeout
                    p.wait(timeout if timeout else None)
                except Exception:
                    p.terminate()
                    if to_file:
                        if tac:
                            with open(to_file, 'wb') as f_dst:
                                tac_logs(_output, f_dst, True)
                    # until we separate timeouts from the `timeout` command
                    # handle per-cmd timeouts via Plugin status checks
                    reader.running = False
                    return {'status': 124, 'output': reader.get_contents(),
                            'truncated': reader.is_full}

            # wait for Popen to set the returncode
            while p.poll() is None:
                pass

            if to_file and tac:
                with open(to_file, 'wb') as f_dst:
                    tac_logs(_output, f_dst,
                             reader.is_full or p.returncode != 0)

            if p.returncode in (126, 127):
                stdout = b""
            else:
                stdout = reader.get_contents()

            return {
                'status': p.returncode,
                'output': stdout,
                'truncated': reader.is_full
            }
    except OSError as e:
        if e.errno == errno.ENOENT:
            return {'status': 127, 'output': "", 'truncated': ''}
        raise e
    finally:
        if hasattr(_output, 'close'):
            _output.close()


def tac_logs(f_src, f_dst, drop_last_log=False):
    """Python implementation of the tac utility with support
    for multiline logs (starting with space). It is intended
    to reverse the output of 'journalctl --reverse'.
    """
    NEWLINE_B = b'\n'
    NEWLINE_I = 10
    SPACE_I = 32
    # make sure all python/libc buffers are flushed
    # else fstat()/mmap() might see partial data
    f_src.flush()
    if os.fstat(f_src.fileno()).st_size == 0:
        return
    with mmap.mmap(f_src.fileno(), 0, access=mmap.ACCESS_READ) as mm:
        sep1 = sep2 = mm.size()-1
        if mm[sep2] != NEWLINE_I:
            drop_last_log = True
        while sep2 >= 0:
            sep1 = mm.rfind(NEWLINE_B, 0, sep1)
            # multiline logs have a first line not starting with space
            # followed by lines starting with spaces
            # line 5
            # line 4
            #  multiline 4
            # line 3
            if mm[sep1+1] == SPACE_I:
                # first line starts with a space
                # (this should not happen)
                if sep1 == -1:
                    break
                # go find the previous NEWLINE
                continue
            # When we truncate or timeout, the last log
            # might be a partial multiline log
            if drop_last_log:
                drop_last_log = False
            else:
                # write the (multi)line log ending with the NEWLINE
                f_dst.write(mm[sep1+1:sep2+1])
            sep2 = sep1


def import_module(module_fqname, superclasses=None):
    """Imports the module module_fqname and returns a list of defined classes
    from that module. If superclasses is defined then the classes returned will
    be subclasses of the specified superclass or superclasses. If superclasses
    is plural it must be a tuple of classes."""
    module_name = module_fqname.rpartition(".")[-1]
    try:
        module = __import__(module_fqname, globals(), locals(), [module_name])
    except ImportError as e:
        print(f'Error while trying to load module {module_fqname}: '
              f' {e.__class__.__name__}')
        raise e
    modules = [class_ for cname, class_ in
               inspect.getmembers(module, inspect.isclass)
               if class_.__module__ == module_fqname]
    if superclasses:
        modules = [m for m in modules if issubclass(m, superclasses)]

    return modules


def shell_out(cmd, timeout=30, chroot=None, runat=None):
    """Shell out to an external command and return the output or the empty
    string in case of error.
    """
    return sos_get_command_output(cmd, timeout=timeout,
                                  chroot=chroot, chdir=runat)['output']


def get_human_readable(size, precision=2):
    # Credit to Pavan Gupta https://stackoverflow.com/questions/5194057/
    suffixes = ['B', 'KiB', 'MiB', 'GiB', 'TiB']
    suffixindex = 0
    while size > 1024 and suffixindex < 4:
        suffixindex += 1
        size = size/1024.0
    return f"{size:.{precision}f}{suffixes[suffixindex]}"


def _os_wrapper(path, sysroot, method, module=os.path):
    if sysroot and sysroot != os.sep:
        if not path.startswith(sysroot):
            path = os.path.join(sysroot, path.lstrip('/'))
    _meth = getattr(module, method)
    return _meth(path)


def path_exists(path, sysroot):
    if '*' in path:
        return _os_wrapper(path, sysroot, 'glob', module=glob)
    return _os_wrapper(path, sysroot, 'exists')


def path_isdir(path, sysroot):
    return _os_wrapper(path, sysroot, 'isdir')


def path_isfile(path, sysroot):
    return _os_wrapper(path, sysroot, 'isfile')


def path_islink(path, sysroot):
    return _os_wrapper(path, sysroot, 'islink')


def listdir(path, sysroot):
    return _os_wrapper(path, sysroot, 'listdir', os)


def path_join(path, *p, sysroot=os.sep):
    if sysroot and not path.startswith(sysroot):
        path = os.path.join(sysroot, path.lstrip(os.sep))
    return os.path.join(path, *p)


def bold(text):
    """Helper to make text bold in console output, without pulling in
    dependencies to the project unneccessarily.

    :param text:    The text to make bold
    :type text:     ``str``

    :returns:       The text wrapped in the ASCII codes to display as bold
    :rtype:         ``str``
    """
    return '\033[1m' + text + '\033[0m'


def recursive_dict_values_by_key(dobj, keys=[]):
    """Recursively compile all elements of a potentially nested dict by a set
    of keys. If a given key is a dict within ``dobj``, then _all_ elements
    within that dict, regardless of child keys, will be returned.

    For example, if a Plugin searches the devices dict for the 'storage' key,
    then all storage devices under the that dict (e.g. block, fibre, etc...)
    will be returned. However, if the Plugin specifies 'block' via ``keys``,
    then only the block devices within the devices['storage'] dict will be
    returned.

    Any elements passed here that are _not_ keys within the dict or any nested
    dicts will also be returned.

    :param dobj:    The 'top-level' dict to intially search by
    :type dobj:     ``dict``

    :param keys:    Which keys to compile elements from within ``dobj``. If no
                    keys are given, all nested elements are returned
    :param keys:    ``list`` of ``str``

    :returns:       All elements within the dict and any nested dicts
    :rtype:         ``list``
    """
    _items = []
    _filt = []
    _items.extend(keys)
    if isinstance(dobj, dict):
        for k, v in dobj.items():
            _filt.append(k)
            # get everything below this key, including nested dicts
            if not keys or k in keys:
                _items.extend(recursive_dict_values_by_key(v))
            # recurse into this dict only for dict keys that match what
            # we're looking for
            elif isinstance(v, dict):
                try:
                    # this will return a nested list, extract it
                    _items.extend(
                        recursive_dict_values_by_key(
                            v[key] for key in keys if key in v
                        )[0]
                    )
                except IndexError:
                    # none of the keys given exist in the nested dict
                    pass
                _filt.extend(v.keys())

    else:
        _items.extend(dobj)

    return [d for d in _items if d not in _filt]


class FakeReader():
    """Used when we are writing directly to disk without sizelimits,
    this allows us to keep more simplified flows for executing,
    monitoring, and collecting command output.
    """

    def __init__(self, process, binary):
        self.process = process
        self.binary = binary

    @property
    def is_full(self):
        return False

    def get_contents(self):
        return '' if not self.binary else b''

    @property
    def running(self):
        return self.process.poll() is None


class HeadReader(threading.Thread):
    """Used to 'head' the command output (f_src) to a given size
    without deadlocking sos. Takes a sizelimit value in MB.
    """

    COPY_BUFSIZE = 1024*1024

    def __init__(self, f_src, f_dst, sizelimit, binary):
        super().__init__()
        self.f_src = f_src
        self.f_dst = f_dst
        self.remaining = sizelimit * 1048576  # convert to bytes
        self.binary = binary
        self.running = True
        self.start()

    def run(self):
        """Reads from the f_src (Popen stdout pipe) until we reach sizelimit.
        once done, close f_src to signal the program that we are done.
        """
        while self.remaining > 0:
            buf = self.f_src.read(min(self.remaining, self.COPY_BUFSIZE))
            if not buf:
                break
            self.f_dst.write(buf)
            self.remaining -= len(buf)
        self.f_src.close()
        self.running = False

    def get_contents(self):
        return '' if not self.binary else b''

    @property
    def is_full(self):
        return self.remaining <= 0


class TailReader(threading.Thread):
    """Used to tail the command output to a given size without deadlocking
    sos.

    Takes a sizelimit value in MB, and will compile stdout from Popen into a
    string that is limited to the given sizelimit.
    """

    def __init__(self, channel, sizelimit, binary):
        super().__init__()
        self.chan = channel
        self.binary = binary
        self.chunksize = 2048
        self.slots = None
        if sizelimit:
            sizelimit = sizelimit * 1048576  # convert to bytes
            self.slots = int(sizelimit / self.chunksize)
        self.deque = deque(maxlen=self.slots)
        self.running = True
        self.start()

    def run(self):
        """Reads from the channel (pipe) that is the output pipe for a
        called Popen. As we are reading from the pipe, the output is added
        to a deque. After the size of the deque exceeds the sizelimit
        earlier (older) entries are removed.

        This means the returned output is chunksize-sensitive, but is not
        really byte-sensitive.
        """
        try:
            while True:
                line = self.chan.read(self.chunksize)
                if not line:
                    # Pipe can remain open after output has completed
                    break
                self.deque.append(line)
        except (ValueError, IOError):
            # pipe has closed, meaning command output is done
            pass
        self.running = False

    def get_contents(self):
        """Returns the contents of the deque as a string"""
        # block until command completes or timesout (separate from the plugin
        # hitting a timeout)
        while self.running:
            time.sleep(0.01)
        if not self.binary:
            return ''.join(ln.decode('utf-8', 'ignore') for ln in self.deque)
        return b''.join(ln for ln in self.deque)

    @property
    def is_full(self):
        """Checks if the deque is full, implying that output was truncated"""
        if not self.slots:
            return False
        return len(self.deque) == self.slots


class ImporterHelper:
    """Provides a list of modules that can be imported in a package.
    Importable modules are located along the module __path__ list and modules
    are files that end in .py.
    """

    def __init__(self, package):
        """package is a package module
        import my.package.module
        helper = ImporterHelper(my.package.module)"""
        self.package = package

    def _plugin_name(self, path):
        "Returns the plugin module name given the path"
        base = os.path.basename(path)
        name, _ = os.path.splitext(base)
        return name

    def _get_plugins_from_list(self, list_):
        plugins = [self._plugin_name(plugin)
                   for plugin in list_
                   if "__init__" not in plugin and plugin.endswith(".py")]
        plugins.sort()
        return plugins

    def _find_plugins_in_dir(self, path):
        if os.path.exists(path):
            py_files = list(find("*.py", path))
            pnames = self._get_plugins_from_list(py_files)
            if pnames:
                return pnames
        return []

    def get_modules(self):
        """Returns the list of importable modules in the configured python
        package. """
        plugins = []
        for path in self.package.__path__:
            if os.path.isdir(path):
                plugins.extend(self._find_plugins_in_dir(path))

        return plugins


class TempFileUtil():

    def __init__(self, tmp_dir):
        self.tmp_dir = tmp_dir
        self.files = []

    def new(self):
        fd, fname = tempfile.mkstemp(dir=self.tmp_dir)
        # avoid TOCTOU race by using os.fdopen()
        fobj = os.fdopen(fd, 'w+')
        self.files.append((fname, fobj))
        return fobj

    def clean(self):
        for fname, f in self.files:
            try:
                f.flush()
                f.close()
            except Exception:
                # file already closed or potentially already removed, ignore
                pass
            try:
                os.unlink(fname)
            except Exception:
                # if the above failed, this is also likely to fail, ignore
                pass
        self.files = []


class SoSTimeoutError(OSError):
    pass

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc. Jamie Bainbridge <jbainbri@redhat.com>
# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class FirewallD(Plugin, RedHatPlugin):

    short_desc = 'Firewall daemon'

    plugin_name = 'firewalld'
    profiles = ('network',)

    packages = ('firewalld',)

    def setup(self):

        self.add_copy_spec("/etc/firewalld/firewalld.conf",
                           tags='firewalld_conf')

        self.add_copy_spec([
            "/etc/firewalld/*.xml",
            "/etc/firewalld/icmptypes/*.xml",
            "/etc/firewalld/services/*.xml",
            "/etc/firewalld/zones/*.xml",
            "/etc/firewalld/ipsets/*.xml",
            "/etc/firewalld/policies/*.xml",
            "/etc/firewalld/helpers/*.xml",
            "/etc/sysconfig/firewalld",
            "/var/log/firewalld",
        ])

        # use a 10s timeout to workaround dbus problems in
        # docker containers.
        self.add_cmd_output([
            "firewall-cmd --direct --get-all-chains",
            "firewall-cmd --direct --get-all-rules",
            "firewall-cmd --direct --get-all-passthroughs",
            "firewall-cmd --get-log-denied",
            "firewall-cmd --list-all-zones",
            "firewall-cmd --permanent --list-all-zones",
            "firewall-cmd --list-all-policies",
            "firewall-cmd --permanent --list-all-policies",
            "firewall-cmd --permanent --direct --get-all-chains",
            "firewall-cmd --permanent --direct --get-all-rules",
            "firewall-cmd --permanent --direct --get-all-passthroughs",
            "firewall-cmd --state"
        ], timeout=10, cmd_as_tag=True)

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class SosExtrasPluginTest(StageTwoReportTest):
    """Ensure that the sos_extras plugin is properly executing command and
    file collections as defined in the sos_extras config file

    :avocado: tags=stagetwo
    """

    files = [('sos_testing.conf', '/etc/sos/extras.d/sos_testing.conf')]
    # rather than only enabling this plugin, make sure the enablement trigger
    # is working
    sos_cmd = '-n logs,networking,devicemapper,filesys,systemd'

    def test_extras_enabled(self):
        self.assertPluginIncluded('sos_extras')

    def test_setup_message_displayed(self):
        self.assertOutputContains('Collecting data from extras file '
                                  '/etc/sos/extras.d/sos_testing.conf')

    def test_extras_config_parsed(self):
        self.assertFileCollected('/etc/fstab')
        self.assertFileCollected('sos_commands/sos_extras/sos_testing.conf'
                                 '/echo_sos_test')



# Copyright (C) 2023 Canonical Ltd.,
#                    David Negreira <david.negreira@canonical.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import re
from sos.report.plugins import Plugin, UbuntuPlugin


class Microk8s(Plugin, UbuntuPlugin):
    """The Microk8s plugin collects the current status of the microk8s
    snap on a Ubuntu machine.

    It will collect logs from journald related to the snap.microk8s
    units as well as run microk8s commands to retrieve the configuration,
    status, version and loaded plugins.
    """

    short_desc = 'The lightweight Kubernetes'
    plugin_name = "microk8s"
    profiles = ('container',)

    packages = ('microk8s',)

    microk8s_cmd = "microk8s"

    def setup(self):
        self.add_journal(units="snap.microk8s.*")

        microk8s_subcmds = [
            'addons repo list',
            'config',
            'ctr plugins ls',
            'ctr plugins ls -d',
            'status',
            'version'
        ]
        self.add_copy_spec([
            "/var/snap/microk8s/current/args/*",
            "/var/snap/microk8s/current/credentials/client.config",
        ])

        self.add_cmd_output([
            f"{self.microk8s_cmd} {subcmd}" for subcmd in microk8s_subcmds
        ])

        crt_dir = "/var/snap/microk8s/current/var/kubernetes/backend"
        dqlite_crt = f"{crt_dir}/cluster.crt"

        self.add_cmd_output(
            f"openssl x509 -in {dqlite_crt} -noout -dates",
        )

        db_path = "/var/snap/microk8s/current/var/kubernetes/backend"

        # Check for inconsistent dqlite db intervals
        self.add_dir_listing(
            db_path,
            suggest_filename="ls_microk8s_dqlite_dir",
        )

        self.add_copy_spec([
            f"{db_path}/info.yaml",
            f"{db_path}/cluster.yaml",
            f"{db_path}/failure-domain",
        ])

        dqlite_bin = "/snap/microk8s/current/bin/dqlite"
        cert = f"{db_path}/cluster.crt"
        key = f"{db_path}/cluster.key"
        servers = f"{db_path}/cluster.yaml"
        dqlite_cmd = f"{dqlite_bin} -c {cert} -k {key} -s file://{servers} k8s"

        queries = [
            {
                "query": ".cluster",
                "suggested_file_suffix": ".cluster",
            },
            {
                "query": ".cluster",
                "opts": ["-f json",],
                "suggested_file_suffix": ".cluster_-f_json",
            },
            {
                "query": ".leader",
                "suggested_file_suffix": ".leader",
            },
        ]

        try:
            with open(servers, 'r', encoding='utf-8') as cluster_definition:
                cluster = cluster_definition.read()
                nodes = re.findall(
                    r'Address:\s*(\d+\.\d+\.\d+\.\d+:\d+)', cluster
                )

                for node in nodes:
                    queries.append({
                        "query": f".describe {node}",
                        "suggested_file_suffix": f".describe_{node}",
                        "opts": ["-f json",],
                    })

        except Exception as e:
            self.add_alert(f"Failed to parse {servers}: {e}")

        for query_entry in queries:
            sql_cmd = dqlite_cmd
            opts = query_entry.get("opts", [])
            for opt in opts:
                sql_cmd = f"{sql_cmd} {opt}"

            query = json.dumps(query_entry.get("query"))
            file_suffix = query_entry.get("suggested_file_suffix")
            self.add_cmd_output(
                f"{sql_cmd} {query}",
                suggest_filename=f"microk8s_sql_{file_suffix}",
            )

    def postproc(self):
        rsub = r'(certificate-authority-data:|token:)\s.*'
        self.do_cmd_output_sub(self.microk8s_cmd, rsub, r'\1 "**********"')

        protect_keys = [
            "certificate-authority-data",
            "client-certificate-data",
            "client-key-data",
            "token",
        ]

        key_regex = fr'(^\s*({"|".join(protect_keys)})\s*:\s*)(.*)'

        self.do_path_regex_sub(
            "/var/snap/microk8s/current/credentials/client.config",
            key_regex, r"\1*********"
        )

# vim: set et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageTwoReportTest


class rhbz1950350(StageTwoReportTest):
    """Ensure that when `--clean` is used with report that the config settings
    from sos.conf under the [clean] section are loaded as well

    :avocado: tags=stagetwo
    """

    files = [
        ('sos.conf', '/etc/sos/sos.conf'),
        ('sos_clean_config.conf', '/etc/sos/extras.d/sos_clean_config.conf'),
        ('clean_config_test.txt', '/var/log/clean_config_test.txt')
    ]

    sos_cmd = '-v -o sos_extras --clean'

    def test_clean_config_loaded(self):
        self.assertSosLogContains(
            "effective options now: (.*)? --clean --domains (.*)? "
            "--keywords (.*)?"
        )

    def test_clean_config_performed(self):
        self.assertFileCollected('var/log/clean_config_test.txt')
        self.assertFileHasContent(
            'var/log/clean_config_test.txt',
            'The domain example.com should not be removed.'
        )
        self.assertFileNotHasContent(
            'var/log/clean_config_test.txt',
            "This line contains 'shibboleth' which should be scrubbed."
        )
        self.assertFileNotHasContent(
            'var/log/clean_config_test.txt',
            'sosexample.com'
        )



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Ebpf(Plugin, IndependentPlugin):

    short_desc = 'eBPF tool'
    plugin_name = 'ebpf'
    profiles = ('system', 'kernel', 'network')
    commands = ('bpftool',)

    option_list = [
        PluginOpt("namespaces", default=None, val_type=int,
                  desc="Number of namespaces to collect, 0 for unlimited"),
    ]

    def get_bpftool_prog_ids(self, prog_json):
        """ Collect the list of program IDs """
        out = []
        try:
            prog_data = json.loads(prog_json)
        except Exception as err:  # pylint: disable=broad-except
            self._log_info(f"Couldn't parse bpftool prog list: {err}")
            return out
        for _, item in enumerate(prog_data):
            if "id" in item:
                out.append(item["id"])
        return out

    def get_bpftool_map_ids(self, map_json):
        """ Collect the list of mapIDs """
        out = []
        try:
            map_data = json.loads(map_json)
        except Exception as err:  # pylint: disable=broad-except
            self._log_info(f"Could not parse bpftool map list: {err}")
            return out
        for _, item in enumerate(map_data):
            if "id" in item:
                out.append(item["id"])
        return out

    def setup(self):
        # collect list of eBPF programs and maps and their dumps
        progs = self.collect_cmd_output("bpftool -j prog list")
        for prog_id in self.get_bpftool_prog_ids(progs['output']):
            for dumpcmd in ["xlated", "jited"]:
                self.add_cmd_output(f"bpftool prog dump {dumpcmd} id "
                                    f"{prog_id}")

        maps = self.collect_cmd_output("bpftool -j map list")
        for map_id in self.get_bpftool_map_ids(maps['output']):
            self.add_cmd_output(f"bpftool map dump id {map_id}")

        self.add_cmd_output([
            # collect list of eBPF programs and maps and their dumps
            # in human readable form
            "bpftool prog list",
            "bpftool map list",
            # Iterate over all cgroups and list all attached programs
            "bpftool cgroup tree",
            # collect list of bpf program attachments in the kernel
            # networking subsystem
            "bpftool net list",
            # collect all struct_ops currently existing in the system
            "bpftool struct_ops dump"
        ])

        # Capture list of bpf program attachments from namespaces
        cmd_prefix = "ip netns exec "
        nsps = self.get_option('namespaces')
        for namespace in self.get_network_namespaces(ns_max=nsps):
            ns_cmd_prefix = cmd_prefix + namespace + " "
            self.add_cmd_output(ns_cmd_prefix + "bpftool net list")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, IndependentPlugin


class Nvidia(Plugin, IndependentPlugin):

    short_desc = 'Nvidia GPU information'
    plugin_name = 'nvidia'
    commands = ('nvidia-smi', 'nvidia-ctk',)
    services = ('nvidia-persistenced', 'nvidia-fabricmanager',
                'nvidia-toolkit-firstboot')

    def setup(self):
        self.add_copy_spec("/etc/cdi/nvidia.yaml")

        subcmds = [
            '--list-gpus',
            '-q -d PERFORMANCE',
            '-q -d SUPPORTED_CLOCKS',
            '-q -d PAGE_RETIREMENT',
            '-q',
            '-q -d ECC',
            'nvlink -s',
            'nvlink -e'
        ]
        ctk_subcmds = [
            'cdi list',
            '--version',
        ]
        self.add_cmd_output([f"nvidia-smi {cmd}" for cmd in subcmds])
        self.add_cmd_output([f"nvidia-ctk {cmd}" for cmd in ctk_subcmds])

        query = ('gpu_name,gpu_bus_id,vbios_version,temperature.gpu,'
                 'utilization.gpu,memory.total,memory.free,memory.used,'
                 'clocks.applications.graphics,clocks.applications.memory')
        querypages = ('timestamp,gpu_bus_id,gpu_serial,gpu_uuid,'
                      'retired_pages.address,retired_pages.cause')
        self.add_cmd_output(f"nvidia-smi --query-gpu={query} --format=csv")
        self.add_cmd_output(
            f"nvidia-smi --query-retired-pages={querypages} --format=csv"
        )

# vim: set et ts=4 sw=4 :
